@@ -37,26 +37,26 @@
 """
 from .config import driver_config, ServerConfig, DatabaseConfig, DriverConfig
 from .fbapi import load_api, get_api
-from .types import Error, InterfaceError, DatabaseError, DataError, \
-     OperationalError, IntegrityError, InternalError, ProgrammingError, \
-     NotSupportedError, FirebirdWarning, \
-     NetProtocol, DirectoryCode, PageSize, DBKeyScope, DbInfoCode, Features, ReplicaMode, \
-     StmtInfoCode, TraInfoCode, TraInfoIsolation, TraInfoReadCommitted, \
-     TraInfoAccess, TraIsolation, TraReadCommitted, TraLockResolution, TraAccessMode, \
-     TableShareMode, TableAccessMode, Isolation, DefaultAction, StatementType, BlobType, \
-     DbAccessMode, DbSpaceReservation, DbWriteMode, ShutdownMode, OnlineMode, \
-     ShutdownMethod, \
-     CancelType, DecfloatRound, DecfloatTraps, \
-     ServerCapability, SrvRepairFlag, SrvStatFlag, SrvBackupFlag, \
-     SrvRestoreFlag, SrvNBackupFlag, SrvInfoCode, ConnectionFlag, EncryptionFlag, \
-     apilevel, threadsafety, paramstyle, DESCRIPTION_NAME, DESCRIPTION_TYPE_CODE, \
-     DESCRIPTION_DISPLAY_SIZE, DESCRIPTION_INTERNAL_SIZE, DESCRIPTION_PRECISION, \
-     DESCRIPTION_SCALE, DESCRIPTION_NULL_OK, Date, Time, Timestamp, DateFromTicks, \
-     TimeFromTicks, TimestampFromTicks, STRING, BINARY, NUMBER, DATETIME, ROWID, \
-     get_timezone
-from .core import connect, create_database, connect_server, transaction, temp_database, \
-     tpb, TPB, CHARSET_MAP, DistributedTransactionManager, Connection, Cursor, \
-     Server, Statement
+from .types import (Error, InterfaceError, DatabaseError, DataError,
+     OperationalError, IntegrityError, InternalError, ProgrammingError,
+     NotSupportedError, FirebirdWarning,
+     NetProtocol, DirectoryCode, PageSize, DBKeyScope, DbInfoCode, Features, ReplicaMode,
+     StmtInfoCode, TraInfoCode, TraInfoIsolation, TraInfoReadCommitted,
+     TraInfoAccess, TraIsolation, TraReadCommitted, TraLockResolution, TraAccessMode,
+     TableShareMode, TableAccessMode, Isolation, DefaultAction, StatementType, BlobType,
+     DbAccessMode, DbSpaceReservation, DbWriteMode, ShutdownMode, OnlineMode,
+     ShutdownMethod,
+     CancelType, DecfloatRound, DecfloatTraps,
+     ServerCapability, SrvRepairFlag, SrvStatFlag, SrvBackupFlag,
+     SrvRestoreFlag, SrvNBackupFlag, SrvInfoCode, ConnectionFlag, EncryptionFlag,
+     apilevel, threadsafety, paramstyle, DESCRIPTION_NAME, DESCRIPTION_TYPE_CODE,
+     DESCRIPTION_DISPLAY_SIZE, DESCRIPTION_INTERNAL_SIZE, DESCRIPTION_PRECISION,
+     DESCRIPTION_SCALE, DESCRIPTION_NULL_OK, Date, Time, Timestamp, DateFromTicks,
+     TimeFromTicks, TimestampFromTicks, STRING, BINARY, NUMBER, DATETIME, ROWID,
+     get_timezone)
+from .core import (connect, create_database, connect_server, transaction, temp_database,
+     tpb, TPB, CHARSET_MAP, DistributedTransactionManager, Connection, Cursor,
+     Server, Statement)
 
 #: Current driver version, SEMVER string.
-__VERSION__ = '1.5.2'
+__VERSION__ = '1.6.0'
